"use strict";

exports.__esModule = true;
exports.default = exports.TagNode = void 0;

var _char = require("./char");

var _index = require("./index");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var getTagAttrs = (tag, params) => {
  var uniqAattr = (0, _index.getUniqAttr)(params);

  if (uniqAattr) {
    var tagAttr = (0, _index.attrValue)(tag, uniqAattr);

    var attrs = _extends({}, params);

    delete attrs[uniqAattr];
    var attrsStr = (0, _index.attrsToString)(attrs);
    return "" + tagAttr + attrsStr;
  }

  return "" + tag + (0, _index.attrsToString)(params);
};

class TagNode {
  constructor(tag, attrs, content) {
    this.tag = tag;
    this.attrs = attrs;
    this.content = Array.isArray(content) ? content : [content];
  }

  attr(name, value) {
    if (typeof value !== 'undefined') {
      this.attrs[name] = value;
    }

    return this.attrs[name];
  }

  append(value) {
    return (0, _index.appendToNode)(this, value);
  }

  get length() {
    return (0, _index.getNodeLength)(this);
  }

  toTagStart(_temp) {
    var {
      openTag = _char.OPEN_BRAKET,
      closeTag = _char.CLOSE_BRAKET
    } = _temp === void 0 ? {} : _temp;
    var tagAttrs = getTagAttrs(this.tag, this.attrs);
    return "" + openTag + tagAttrs + closeTag;
  }

  toTagEnd(_temp2) {
    var {
      openTag = _char.OPEN_BRAKET,
      closeTag = _char.CLOSE_BRAKET
    } = _temp2 === void 0 ? {} : _temp2;
    return "" + openTag + _char.SLASH + this.tag + closeTag;
  }

  toTagNode() {
    return new TagNode(this.tag.toLowerCase(), this.attrs, this.content);
  }

  toString(_temp3) {
    var {
      openTag = _char.OPEN_BRAKET,
      closeTag = _char.CLOSE_BRAKET
    } = _temp3 === void 0 ? {} : _temp3;
    var isEmpty = this.content.length === 0;
    var content = this.content.reduce((r, node) => r + node.toString({
      openTag,
      closeTag
    }), '');
    var tagStart = this.toTagStart({
      openTag,
      closeTag
    });

    if (isEmpty) {
      return tagStart;
    }

    return "" + tagStart + content + this.toTagEnd({
      openTag,
      closeTag
    });
  }

}

exports.TagNode = TagNode;

TagNode.create = function (tag, attrs, content) {
  if (attrs === void 0) {
    attrs = {};
  }

  if (content === void 0) {
    content = [];
  }

  return new TagNode(tag, attrs, content);
};

TagNode.isOf = (node, type) => node.tag === type;

var _default = TagNode;
exports.default = _default;