"use strict";

exports.__esModule = true;
exports["default"] = exports.TagNode = void 0;

var _char = require("./char");

var _index = require("./index");

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var getTagAttrs = function getTagAttrs(tag, params) {
  var uniqAattr = (0, _index.getUniqAttr)(params);

  if (uniqAattr) {
    var tagAttr = (0, _index.attrValue)(tag, uniqAattr);

    var attrs = _extends({}, params);

    delete attrs[uniqAattr];
    var attrsStr = (0, _index.attrsToString)(attrs);
    return "" + tagAttr + attrsStr;
  }

  return "" + tag + (0, _index.attrsToString)(params);
};

var TagNode =
/*#__PURE__*/
function () {
  function TagNode(tag, attrs, content) {
    this.tag = tag;
    this.attrs = attrs;
    this.content = Array.isArray(content) ? content : [content];
  }

  var _proto = TagNode.prototype;

  _proto.attr = function attr(name, value) {
    if (typeof value !== 'undefined') {
      this.attrs[name] = value;
    }

    return this.attrs[name];
  };

  _proto.append = function append(value) {
    return (0, _index.appendToNode)(this, value);
  };

  _proto.toTagStart = function toTagStart(_temp) {
    var _ref = _temp === void 0 ? {} : _temp,
        _ref$openTag = _ref.openTag,
        openTag = _ref$openTag === void 0 ? _char.OPEN_BRAKET : _ref$openTag,
        _ref$closeTag = _ref.closeTag,
        closeTag = _ref$closeTag === void 0 ? _char.CLOSE_BRAKET : _ref$closeTag;

    var tagAttrs = getTagAttrs(this.tag, this.attrs);
    return "" + openTag + tagAttrs + closeTag;
  };

  _proto.toTagEnd = function toTagEnd(_temp2) {
    var _ref2 = _temp2 === void 0 ? {} : _temp2,
        _ref2$openTag = _ref2.openTag,
        openTag = _ref2$openTag === void 0 ? _char.OPEN_BRAKET : _ref2$openTag,
        _ref2$closeTag = _ref2.closeTag,
        closeTag = _ref2$closeTag === void 0 ? _char.CLOSE_BRAKET : _ref2$closeTag;

    return "" + openTag + _char.SLASH + this.tag + closeTag;
  };

  _proto.toTagNode = function toTagNode() {
    return new TagNode(this.tag.toLowerCase(), this.attrs, this.content);
  };

  _proto.toString = function toString(_temp3) {
    var _ref3 = _temp3 === void 0 ? {} : _temp3,
        _ref3$openTag = _ref3.openTag,
        openTag = _ref3$openTag === void 0 ? _char.OPEN_BRAKET : _ref3$openTag,
        _ref3$closeTag = _ref3.closeTag,
        closeTag = _ref3$closeTag === void 0 ? _char.CLOSE_BRAKET : _ref3$closeTag;

    var isEmpty = this.content.length === 0;
    var content = this.content.reduce(function (r, node) {
      return r + node.toString({
        openTag: openTag,
        closeTag: closeTag
      });
    }, '');
    var tagStart = this.toTagStart({
      openTag: openTag,
      closeTag: closeTag
    });

    if (isEmpty) {
      return tagStart;
    }

    return "" + tagStart + content + this.toTagEnd({
      openTag: openTag,
      closeTag: closeTag
    });
  };

  _createClass(TagNode, [{
    key: "length",
    get: function get() {
      return (0, _index.getNodeLength)(this);
    }
  }]);

  return TagNode;
}();

exports.TagNode = TagNode;

TagNode.create = function (tag, attrs, content) {
  if (attrs === void 0) {
    attrs = {};
  }

  if (content === void 0) {
    content = [];
  }

  return new TagNode(tag, attrs, content);
};

TagNode.isOf = function (node, type) {
  return node.tag === type;
};

var _default = TagNode;
exports["default"] = _default;