/// <reference types="node" />
import { EventEmitter } from 'events';
import { HttpClient } from '@doctormckay/stdlib/http';
import EAuthTokenPlatformType from './enums-steam/EAuthTokenPlatformType';
import ITransport from './transports/ITransport';
import { CAuthentication_GetPasswordRSAPublicKey_Response } from './protobuf-generated/types';
import { CheckMachineAuthRequest, CheckMachineAuthResponse, GetAuthSessionInfoRequest, GetAuthSessionInfoResponse, MobileConfirmationRequest, PollLoginStatusRequest, PollLoginStatusResponse, StartAuthSessionRequest, StartAuthSessionWithCredentialsRequest, StartAuthSessionWithCredentialsResponse, StartAuthSessionWithQrResponse, SubmitSteamGuardCodeRequest } from './interfaces-internal';
interface RequestDefinition {
    apiInterface: string;
    apiMethod: string;
    apiVersion: number;
    data: any;
    accessToken?: string;
}
export default class AuthenticationClient extends EventEmitter {
    _transport: ITransport;
    _platformType: EAuthTokenPlatformType;
    _webClient: HttpClient;
    constructor(platformType: EAuthTokenPlatformType, transport: ITransport, webClient: HttpClient);
    getRsaKey(accountName: string): Promise<CAuthentication_GetPasswordRSAPublicKey_Response>;
    encryptPassword(accountName: string, password: string): Promise<{
        encryptedPassword: string;
        keyTimestamp: string;
    }>;
    startSessionWithCredentials(details: StartAuthSessionWithCredentialsRequest): Promise<StartAuthSessionWithCredentialsResponse>;
    startSessionWithQR(details: StartAuthSessionRequest): Promise<StartAuthSessionWithQrResponse>;
    submitSteamGuardCode(details: SubmitSteamGuardCodeRequest): Promise<void>;
    checkMachineAuthOrSendCodeEmail(details: CheckMachineAuthRequest): Promise<CheckMachineAuthResponse>;
    pollLoginStatus(details: PollLoginStatusRequest): Promise<PollLoginStatusResponse>;
    getAuthSessionInfo(accessToken: string, details: GetAuthSessionInfoRequest): Promise<GetAuthSessionInfoResponse>;
    submitMobileConfirmation(accessToken: string, details: MobileConfirmationRequest): Promise<void>;
    generateAccessTokenForApp(refreshToken: string, renewRefreshToken?: boolean): Promise<{
        accessToken: string;
        refreshToken?: string;
    }>;
    sendRequest(request: RequestDefinition): Promise<any>;
    close(): void;
}
export {};
//# sourceMappingURL=AuthenticationClient.d.ts.map